/* Copyright (c) 2013 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
 *
 * @defgroup ble_sdk_app_ant_hrs_main main.c
 * @{
 * @ingroup ble_sdk_app_ant_hrs
 * @brief HRM sample application using both BLE and ANT.
 *
 * The application uses the BLE Heart Rate Service (and also the Device Information
 * services), and the ANT HRM RX profile.
 *
 * It will open a receive channel which will connect to an ANT HRM TX profile device when the
 * application starts. The received data will be propagated to a BLE central through the
 * BLE Heart Rate Service.
 *
 * The ANT HRM TX profile device simulator SDK application
 * (Board\pca10003\ant\ant_hrm\hrm_tx_buttons) can be used as a peer ANT device. By changing
 * ANT_HRMRX_NETWORK_KEY to the ANT+ Network Key, the application will instead be able to connect to
 * an ANT heart rate belt.
 *
 * @note The ANT+ Network Key is available for ANT+ Adopters. Please refer to
 *       http://thisisant.com to become an ANT+ Adopter and access the key.
 *
 * @note This application is based on the BLE Heart Rate Service Sample Application
 *       (Board\nrf6310\ble\ble_app_hrs). Please refer to this application for additional
 *       documentation.
 */
//----------------------------------FALCO DEVELOPMENT-28/01/2016---------------------------------------

/** @file
 *
 * @defgroup ble_sdk_app_ant_hrs_main main.c
 * @{
 * @ingroup ble_sdk_app_ant_hrs
 * @brief FALCO-E-MOTORS application using both BLE and ANT.
 *
 * The application uses the BLE FALCO-E-MOTOR Service (and also the Device Information
 * services), and the ANT FALCO-E-MOTORS RX-TX profile.
 *
 * It will open a trans-receive channel which will connect to an ANT falco-e-motors TX-RX profile device when the
 * application starts. The received data will be propagated to a ANT+ and BLE central through the
 * ANT+ and BLE Falco-e-motor Service.
 *
 *Opening of the CH0 of ANT+ device in TX_RX mode to receive and Transmit data from and to a Falco-Console and Falco-GUI
 *Opening of the CH1 of ANT+ device in TX mode to transmit TS data from Falco-e-Motor 
 *Opening of the BLE CH in transreceive mode to tranmit data from falco-e-Motor and receive data for Falco-e-Motor  
 */




#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include "app_uart.h"
#include "app_error.h"
#include "nrf_delay.h"
#include "nordic_common.h"
#include "nrf.h"
#include "app_error.h"
#include "nrf_gpio.h"
#include "nrf51_bitfields.h"
#include "ble.h"
#include "ble_hci.h"
#include "ble_srv_common.h"
#include "ble_advdata.h"
#include "ble_hrs.h"
#include "ble_dis.h"
#include "ble_conn_params.h"
#include "ble_nus.h"
#include "boards.h"
#include "sensorsim.h"
#include "softdevice_handler.h"
#include "app_timer.h"
#include "device_manager.h"
#include "ant_parameters.h"
#include "ant_interface.h"
#include "pstorage.h"
#include "app_trace.h"
#include "bsp.h"
#include "ant_error.h"
#include "ant_stack_config.h"
#include "ant_stack_handler_types.h"




//#include "nrf_drv_rtc.h"
//#include "nrf_drv_clock.h"





#define WAKEUP_BUTTON_ID               	0                                            /**< Button used to wake up the application. */
#define BOND_DELETE_ALL_BUTTON_ID       1                                            /**< Button used for deleting all bonded centrals during startup. */

#define DEVICE_NAME                     "Nordic_UART"                               /**< Name of device. Will be included in the advertising data. */

//#define DEVICE_NAME                     "nrf51422_HRM"                               /**< Name of device. Will be included in the advertising data. */
#define MANUFACTURER_NAME               "NordicSemiconductor"                        /**< Manufacturer. Will be passed to Device Information Service. */
#define APP_ADV_INTERVAL                40                                           /**< The advertising interval (in units of 0.625 ms. This value corresponds to 25 ms). */
#define APP_ADV_TIMEOUT_IN_SECONDS      180                                          /**< The advertising timeout in units of seconds. */
#define NUS_SERVICE_UUID_TYPE           BLE_UUID_TYPE_VENDOR_BEGIN                  /**< UUID type for the Nordic UART Service (vendor specific). */

#define IS_SRVC_CHANGED_CHARACT_PRESENT 0                                            /**< Whether or not to include the service_changed characteristic. If not enabled, the server's database cannot be changed for the lifetime of the device */
#define APP_TIMER_PRESCALER             0                                            /**< Value of the RTC1 PRESCALER register. */
#define APP_TIMER_MAX_TIMERS            (2 + BSP_APP_TIMERS_NUMBER)                  /**< Maximum number of simultaneously created timers. */
#define APP_TIMER_OP_QUEUE_SIZE         4                                            /**< Size of timer operation queues. */

#define SECOND_1_25_MS_UNITS            800                                          /**< Definition of 1 second, when 1 unit is 1.25 ms. */
#define SECOND_10_MS_UNITS              100                                          /**< Definition of 1 second, when 1 unit is 10 ms. */
#define MIN_CONN_INTERVAL               (SECOND_1_25_MS_UNITS / 2)                   /**< Minimum acceptable connection interval (0.5 seconds), Connection interval uses 1.25 ms units. */
#define MAX_CONN_INTERVAL               (SECOND_1_25_MS_UNITS)                       /**< Maximum acceptable connection interval (1 second), Connection interval uses 1.25 ms units. */
#define SLAVE_LATENCY                   0                                            /**< Slave latency. */
#define CONN_SUP_TIMEOUT                (4 * SECOND_10_MS_UNITS)                     /**< Connection supervisory timeout (4 seconds), Supervision Timeout uses 10 ms units. */

#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(5000, APP_TIMER_PRESCALER)   /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds). */
#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(30000, APP_TIMER_PRESCALER)  /**< Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds). */
#define MAX_CONN_PARAMS_UPDATE_COUNT    3                                            /**< Number of attempts before giving up the connection parameter negotiation. */

#define SEC_PARAM_TIMEOUT               30                                           /**< Timeout for Pairing Request or Security Request (in seconds). */
#define SEC_PARAM_BOND                  1                                            /**< Perform bonding. */
#define SEC_PARAM_MITM                  0                                            /**< Man In The Middle protection not required. */
#define SEC_PARAM_IO_CAPABILITIES       BLE_GAP_IO_CAPS_NONE                         /**< No I/O capabilities. */
#define SEC_PARAM_OOB                   0                                            /**< Out Of Band data not available. */
#define SEC_PARAM_MIN_KEY_SIZE          7                                            /**< Minimum encryption key size. */
#define SEC_PARAM_MAX_KEY_SIZE          16                                           /**< Maximum encryption key size. */

#define DEAD_BEEF                       0xDEADBEEF                                   /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */

#define ANT_HRMRX_ANT_CHANNEL           0                                            /**< Default ANT Channel. */
#define ANT_HRMRX_CHANNEL_TYPE          0x40                                         /**< Channel Type Slave RX only. */
#define ANT_HRMRX_DEVICE_TYPE           0x78                                         /**< Channel ID device type. */
#define ANT_HRMRX_DEVICE_NUMBER         0                                            /**< Device Number. */
#define ANT_HRMRX_TRANS_TYPE            0                                            /**< Transmission Type. */
#define ANT_HRMRX_MSG_PERIOD            0x1F86                                       /**< Message Periods, decimal 8070 (4.06Hz). */
#define ANT_HRMRX_EXT_ASSIGN            0x00                                         /**< ANT Ext Assign. */
#define ANT_HRM_TOGGLE_MASK             0x80                                         /**< HRM Page Toggle Bit Mask. */
#define ANTPLUS_NETWORK_NUMBER          0                                            /**< Network number. */
#define ANTPLUS_RF_FREQ                 0x39                                         /**< Frequency, Decimal 57 (2457 MHz). */
#define ANT_HRM_PAGE_0                  0                                            /**< HRM page 0 constant. */
#define ANT_HRM_PAGE_1                  1                                            /**< HRM page 1 constant. */
#define ANT_HRM_PAGE_2                  2                                            /**< HRM page 2 constant. */
#define ANT_HRM_PAGE_3                  3                                            /**< HRM page 3 constant. */
#define ANT_HRM_PAGE_4                  4                                            /**< HRM page 4 constant. */
#define ANT_BUFFER_INDEX_MESG_ID        0x01                                         /**< Index for Message ID. */
#define ANT_BUFFER_INDEX_MESG_DATA      0x03                                         /**< Index for Data. */
//#define ANT_HRMRX_NETWORK_KEY        {0, 0, 0, 0, 0, 0, 0, 0}                     /**< The default network key used. */
#define ANT_FALCO_NETWORK_KEY           {0XB9, 0XA5, 0X21, 0XFB, 0XBD, 0X72, 0XC3, 0X45} /**< The default network key used. */
#define ANT_FALCOTX_ANT_CHANNEL 				0																						/**<CH0 of the ANT device for Motor data transmission.*/
#define ANT_FALCOTX_CHANNEL_TYPE        0x10                                        /**< Channel Type MASTER-TX & RX only. */
#define ANT_FALCOTX_DEVICE_TYPE         1                                         	/**< Channel ID device type. */
#define ANT_FALCOTX_DEVICE_NUMBER       0X10                                        /**< Device Number 20. */
#define ANT_FALCOTX_TRANS_TYPE          1                                           /**< Transmission Type. */
#define ANT_FALCOTX_MSG_PERIOD          0x2000                                      /**< Message Periods, decimal 8192 (4HZ). */
#define ANT_FALCOTX_EXT_ASSIGN          0x00                                        /**< ANT Ext Assign. */
#define ANTPLUS_FALCONETWORK_NUMBER     0                                            /**< Network number. */
#define ANTPLUS_FALCOMRF_FREQ           0x42                                         /**< Frequency, Decimal 66 (2466 MHz). */

#define ANT_FALCOTTX_ANT_CHANNEL 				1																						/**<CH2 of the ANT device for TS transmission.*/
#define ANT_FALCOTTX_CHANNEL_TYPE       0x50                                        /**< Channel Type MASTER-TX only. */
#define ANT_FALCOTTX_DEVICE_TYPE        1                                         	/**< Channel ID device type. */
#define ANT_FALCOTTX_DEVICE_NUMBER      0X20                                        /**< Device Number 0. */
#define ANT_FALCOTTX_TRANS_TYPE         1                                           /**< Transmission Type. */
#define ANT_FALCOTTX_MSG_PERIOD         0x1F96                                      /**< Message Periods, decimal 8087 (4.052HZ). */
#define ANT_FALCOTTX_EXT_ASSIGN         0x00                                        /**< ANT Ext Assign. */
#define ANTPLUS_FALCONETWORK_NUMBER     0                                            /**< Network number. */
#define ANTPLUS_FALCOTRF_FREQ           0x39                                         /**< Frequency, Decimal 57 (2457 MHz). */



//#define ENABLE_LOOPBACK_TEST  /**< if defined, then this example will be a loopback test, which means that TX should be connected to RX to get data loopback. */

#define MAX_TEST_DATA_BYTES     (15U)                /**< max number of test bytes to be used for tx and rx. */
#define UART_TX_BUF_SIZE 256                         /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE 1                           /**< UART RX buffer size. */

#define RX_PIN_NUMBER    12  												/**<	P012 PIN OF N51422 MODULE. */  	
#define TX_PIN_NUMBER    15 												/**<  P015 PIN OF N51422 MODULE. */

//#define RX_PIN_NUMBER    3  												/**<	P03 PIN OF N51422 MODULE. */  	
//#define TX_PIN_NUMBER    5 												/**<  P05 PIN OF N51422 MODULE. */

#define RTS_PIN_NUMBER   5 													/**<  P005 PIN OF N51422 MODULE. */
#define CTS_PIN_NUMBER   4


//PA Module Pin Selection
    #define ANT_SEL       17										/**< Antenna selection for Chip or UFL */
    #define CSD           18										/**< Matching Network selection*/
    #define CPS           19
		#define CHIP_ANTENNA  1											/**< Select the CHIP Antenna*/


// Miscellaneous defines. 
#define ANT_CHANNEL_DEFAULT_NETWORK     0x00                 /**< ANT Channel Network. */
#define ANT_EVENT_MSG_BUFFER_MIN_SIZE   32u                  /**< Minimum size of ANT event message buffer. */
#define BROADCAST_DATA_BUFFER_SIZE      8u                   /**< Size of the broadcast data buffer. */

// Static variables and buffers. 
static uint8_t m_broadcast_data[BROADCAST_DATA_BUFFER_SIZE]; /**< Primary data transmit buffer. */
static uint8_t m_counter = 1u;                               /**< Counter to increment the ANT broadcast data payload. */


static uint16_t                         m_conn_handle = BLE_CONN_HANDLE_INVALID;     /**< Handle of the current connection. */
static ble_gap_adv_params_t             m_adv_params;                                /**< Parameters to be passed to the stack when starting advertising. */
static ble_hrs_t                        m_hrs;                                       /**< Structure used to identify the heart rate service. */
//static uint8_t                        m_ant_network_key[] = ANT_HRMRX_NETWORK_KEY; /**< ANT PLUS network key. */
static uint8_t                          m_ant_network_key[] = ANT_FALCO_NETWORK_KEY; /**< ANT PLUS network key. */

//---variable for the UART-BLE connection--------------------

static ble_nus_t                        m_nus;                                      /**< Structure to identify the Nordic UART Service. */
static ble_uuid_t                       m_adv_uuids[] = {{BLE_UUID_NUS_SERVICE, NUS_SERVICE_UUID_TYPE}};  /**< Universally unique service identifier. */
//static uint16_t                       m_conn_handle = BLE_CONN_HANDLE_INVALID;    /**< Handle of the current connection. */

static uint8_t data_ln=8,data_ln1=15;

uint8_t event,Rx_data,m_broadcast_data_new[20];
uint8_t ant_channel=0;
static uint8_t  falcotx_data ; // Data Recive from the Hx_M motor 
static uint8_t * falcorx_data=(uint8_t *)("123456789abcde");//("\n\rLOOPBACK_TEST\n\r");    // Data send to Hx_M motor 
static uint8_t  falcorx_datasend[15]={1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
static uint8_t  message_head[1]={0xa4};
//uint8_t  data_array[15];
uint8_t	data_arrayrx[15],m_rx_byter1[8],m_rx_byter2[8],cheaksum,l_power;
uint8_t AntDevID_New,TsDevID_New;
uint8_t Set_ID[9]={0xa4,0x99,0x1,0x00,0x00,0x00,0x00,0x00,0x00};

volatile extern uint8_t m_rx_byter[9],EndOfrx_falg;

#ifdef BONDING_ENABLE
static dm_application_instance_t        m_app_handle;                                /**< Application identifier allocated by device manager */
static bool                             m_app_initialized   = false;                 /**< Application initialized flag. */
#endif // BONDING_ENABLE

/**@brief Callback function for asserts in the SoftDevice.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyze
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in]   line_num   Line number of the failing ASSERT call.
 * @param[in]   file_name  File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}


/** @brief Function starting the internal LFCLK XTAL oscillator.
 */
/*static void lfclk_config(void)
{
    ret_code_t err_code = nrf_drv_clock_init(NULL);
    APP_ERROR_CHECK(err_code);

    nrf_drv_clock_lfclk_request();
}*/

/**@brief Start advertising.
 */
static void advertising_start(void)
{
    uint32_t err_code;
    
    err_code = sd_ble_gap_adv_start(&m_adv_params);
    if (err_code != NRF_SUCCESS && err_code != NRF_ERROR_INVALID_STATE)
    {
        APP_ERROR_HANDLER(err_code);
    }
    
    err_code = bsp_indication_set(BSP_INDICATE_ADVERTISING);
    APP_ERROR_CHECK(err_code);
}


/**@brief Start receiving the ANT HRM data.
 */
static void ant_hrm_rx_start(void)
{
    uint32_t err_code = sd_ant_channel_open(ANT_HRMRX_ANT_CHANNEL);
    if (err_code != NRF_SUCCESS && err_code != NRF_ANT_ERROR_CHANNEL_IN_WRONG_STATE)
    {
        APP_ERROR_HANDLER(err_code);
    }
}


/**@brief Attempt to both open the ant channel and start ble advertising.
*/
static void ant_and_adv_start(void)
{
    advertising_start(); // start transmiiting data via BLE 
   // ant_hrm_rx_start(); // received data from the ANT device  
}


/**@brief Timer initialization.
 *
 * @details Initializes the timer module. This creates and starts application timers.
 */
static void timers_init(void)
{
    // Initialize timer module
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_MAX_TIMERS, APP_TIMER_OP_QUEUE_SIZE, NULL);
}


/**@brief GAP initialization.
 *
 * @details This function sets up all the necessary GAP (Generic Access Profile) parameters of the
 *          device including the device name, appearance, and the preferred connection parameters.
 */
/*static void gap_params_init(void)
{
    uint32_t                err_code;
    ble_gap_conn_params_t   gap_conn_params;
    ble_gap_conn_sec_mode_t sec_mode;

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);
    
    err_code = sd_ble_gap_device_name_set(&sec_mode, (const uint8_t *)DEVICE_NAME, strlen(DEVICE_NAME));
    APP_ERROR_CHECK(err_code);

    err_code = sd_ble_gap_appearance_set(BLE_APPEARANCE_HEART_RATE_SENSOR_HEART_RATE_BELT);
    APP_ERROR_CHECK(err_code);
    
    memset(&gap_conn_params, 0, sizeof(gap_conn_params));

    gap_conn_params.min_conn_interval = MIN_CONN_INTERVAL;
    gap_conn_params.max_conn_interval = MAX_CONN_INTERVAL;
    gap_conn_params.slave_latency     = SLAVE_LATENCY;
    gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;

    err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
    APP_ERROR_CHECK(err_code);
}*/

static void gap_params_init(void)
{
    uint32_t                err_code;
    ble_gap_conn_params_t   gap_conn_params;
    ble_gap_conn_sec_mode_t sec_mode;

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);
    
    err_code = sd_ble_gap_device_name_set(&sec_mode,
                                          (const uint8_t *) DEVICE_NAME,
                                          strlen(DEVICE_NAME));
    APP_ERROR_CHECK(err_code);

    memset(&gap_conn_params, 0, sizeof(gap_conn_params));

    gap_conn_params.min_conn_interval = MIN_CONN_INTERVAL;
    gap_conn_params.max_conn_interval = MAX_CONN_INTERVAL;
    gap_conn_params.slave_latency     = SLAVE_LATENCY;
    gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;

    err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
    APP_ERROR_CHECK(err_code);
}


/**@brief Advertising functionality initialization.
 *
 * @details Encodes the required advertising data and passes it to the stack.
 *          Also builds a structure to be passed to the stack when starting advertising.
 */
static void advertising_init(void)
{
    uint32_t      err_code;
    ble_advdata_t advdata;
    uint8_t       flags = BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE;
    
   /* ble_uuid_t adv_uuids[] =
    {
        {BLE_UUID_HEART_RATE_SERVICE,         BLE_UUID_TYPE_BLE},
        {BLE_UUID_DEVICE_INFORMATION_SERVICE, BLE_UUID_TYPE_BLE}
    };*/
	
		ble_uuid_t adv_uuids[] =
    {
        {BLE_UUID_NUS_SERVICE,        BLE_UUID_TYPE_BLE},
        {BLE_UUID_DEVICE_INFORMATION_SERVICE,BLE_UUID_TYPE_BLE}
    };
	
	
		

    // Build and set advertising data.
    memset(&advdata, 0, sizeof(advdata));
    
    advdata.name_type               = BLE_ADVDATA_FULL_NAME;
    advdata.include_appearance      = true;
    advdata.flags                   = flags;
    advdata.uuids_complete.uuid_cnt = sizeof(adv_uuids) / sizeof(adv_uuids[0]);
    advdata.uuids_complete.p_uuids  = adv_uuids;
		//advdata.uuids_complete.uuid_cnt = sizeof(m_adv_uuids) / sizeof(m_adv_uuids[0]);
    //advdata.uuids_complete.p_uuids  = m_adv_uuids;
    
    err_code = ble_advdata_set(&advdata, NULL);
    APP_ERROR_CHECK(err_code);

    // Initialise advertising parameters (used when starting advertising).
    memset(&m_adv_params, 0, sizeof(m_adv_params));
    
    m_adv_params.type        = BLE_GAP_ADV_TYPE_ADV_IND;
    m_adv_params.p_peer_addr = NULL;
    m_adv_params.fp          = BLE_GAP_ADV_FP_ANY;
    m_adv_params.interval    = APP_ADV_INTERVAL;
    m_adv_params.timeout     = APP_ADV_TIMEOUT_IN_SECONDS;
}

/*static void advertising_init(void)
{
    uint32_t      err_code;
    ble_advdata_t advdata;
    ble_advdata_t scanrsp;

    // Build advertising data struct to pass into @ref ble_advertising_init.
    memset(&advdata, 0, sizeof(advdata));
    advdata.name_type          = BLE_ADVDATA_FULL_NAME;
    advdata.include_appearance = false;
    advdata.flags              = BLE_GAP_ADV_FLAGS_LE_ONLY_LIMITED_DISC_MODE;

		memset(&scanrsp, 0, sizeof(scanrsp));
    scanrsp.uuids_complete.uuid_cnt = sizeof(m_adv_uuids) / sizeof(m_adv_uuids[0]);
    scanrsp.uuids_complete.p_uuids  = m_adv_uuids;

   // ble_adv_modes_config_t options = {0};
    options.ble_adv_fast_enabled  = BLE_ADV_FAST_ENABLED;
    options.ble_adv_fast_interval = APP_ADV_INTERVAL;
    options.ble_adv_fast_timeout  = APP_ADV_TIMEOUT_IN_SECONDS;//

    //err_code = ble_advertising_init(&advdata, &scanrsp, &options, on_adv_evt, NULL);
    //APP_ERROR_CHECK(err_code);
		
		// Initialise advertising parameters (used when starting advertising).
    memset(&m_adv_params, 0, sizeof(m_adv_params));
    
    m_adv_params.type        = BLE_GAP_ADV_TYPE_ADV_IND;
    m_adv_params.p_peer_addr = NULL;
    m_adv_params.fp          = BLE_GAP_ADV_FP_ANY;
    m_adv_params.interval    = APP_ADV_INTERVAL;
    m_adv_params.timeout     = APP_ADV_TIMEOUT_IN_SECONDS;
    
     
		
}*/



/**@brief Initialize services that will be used by the application.
 *
 * @details Initialize the Heart Rate and Device Information services.
 */
 /*
static void services_init(void)
{
    uint32_t       err_code;
    ble_hrs_init_t hrs_init;
    ble_dis_init_t dis_init;
    uint8_t        body_sensor_location;
    
    // Initialize Heart Rate Service.
    body_sensor_location = BLE_HRS_BODY_SENSOR_LOCATION_FINGER;
    
    memset(&hrs_init, 0, sizeof(hrs_init));
    
    hrs_init.evt_handler                 = NULL;
    hrs_init.is_sensor_contact_supported = false;
    hrs_init.p_body_sensor_location      = &body_sensor_location;

    // Here the sec level for the Heart Rate Service can be changed/increased.
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&hrs_init.hrs_hrm_attr_md.cccd_write_perm);
    BLE_GAP_CONN_SEC_MODE_SET_NO_ACCESS(&hrs_init.hrs_hrm_attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_NO_ACCESS(&hrs_init.hrs_hrm_attr_md.write_perm);

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&hrs_init.hrs_bsl_attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_NO_ACCESS(&hrs_init.hrs_bsl_attr_md.write_perm);

    err_code = ble_hrs_init(&m_hrs, &hrs_init);
    APP_ERROR_CHECK(err_code);

    // Initialize Device Information Service
    memset(&dis_init, 0, sizeof(dis_init));
    
    ble_srv_ascii_to_utf8(&dis_init.manufact_name_str, MANUFACTURER_NAME);

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&dis_init.dis_attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_NO_ACCESS(&dis_init.dis_attr_md.write_perm);

    err_code = ble_dis_init(&dis_init);
    APP_ERROR_CHECK(err_code);
}*/

/**@brief Function for handling the data from the Nordic UART Service.
 *
 * @details This function will process the data received from the Nordic UART BLE Service and send
 *          it to the UART module.
 *
 * @param[in] p_nus    Nordic UART Service structure.
 * @param[in] p_data   Data to be send to UART module.
 * @param[in] length   Length of the data.
 */
/**@snippet [Handling the data received over BLE] */
static void nus_data_handler(ble_nus_t * p_nus, uint8_t * p_data, uint16_t length)
{
		while(app_uart_put(message_head[0]) != NRF_SUCCESS); // transmit data via UART
	
    for (uint32_t i = 0; i <length; i++)
    {
        while(app_uart_put(p_data[i]) != NRF_SUCCESS);
				//nrf_delay_ms(50);
    }
    //while(app_uart_put('\n') != NRF_SUCCESS);
}
/**@snippet [Handling the data received over BLE] */

static void services_init(void)
{
    uint32_t       err_code;
    ble_nus_init_t nus_init;
		ble_dis_init_t dis_init;
    
    memset(&nus_init, 0, sizeof(nus_init));

    nus_init.data_handler = nus_data_handler;
    
    err_code = ble_nus_init(&m_nus, &nus_init);
    APP_ERROR_CHECK(err_code);
	
	//---------------------------------------------------------------------
	    // Initialize Device Information Service
    memset(&dis_init, 0, sizeof(dis_init));
	    
		ble_srv_ascii_to_utf8(&dis_init.manufact_name_str, MANUFACTURER_NAME);

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&dis_init.dis_attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_NO_ACCESS(&dis_init.dis_attr_md.write_perm);

    err_code = ble_dis_init(&dis_init);
    APP_ERROR_CHECK(err_code);
}


/**@brief Connection Parameters Module handler.
 *
 * @details This function will be called for all events in the Connection Parameters Module which
 *          are passed to the application.
 *          @note All this function does is to disconnect. This could have been done by simply
 *                setting the disconnect_on_fail config parameter, but instead we use the event
 *                handler mechanism to demonstrate its use.
 *
 * @param[in]   p_evt   Event received from the Connection Parameters Module.
 */
static void on_conn_params_evt(ble_conn_params_evt_t * p_evt)
{
    uint32_t err_code;
    
    switch (p_evt->evt_type)
    {
        case BLE_CONN_PARAMS_EVT_FAILED:
            err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_CONN_INTERVAL_UNACCEPTABLE);
            APP_ERROR_CHECK(err_code);
            break;
            
        default:
            // No implementation needed.
            break;
    }
}


/**@brief Connection Parameters module error handler.
 *
 * @param[in]   nrf_error   Error code containing information about what went wrong.
 */
static void conn_params_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}


/**@brief Initialize the Connection Parameters module.
 */
/*static void conn_params_init(void)
{
    uint32_t               err_code;
    ble_conn_params_init_t cp_init;

    memset(&cp_init, 0, sizeof(cp_init));

    cp_init.p_conn_params                  = NULL;
    cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
    cp_init.next_conn_params_update_delay  = NEXT_CONN_PARAMS_UPDATE_DELAY;
    cp_init.max_conn_params_update_count   = MAX_CONN_PARAMS_UPDATE_COUNT;
    cp_init.start_on_notify_cccd_handle    = m_hrs.hrm_handles.cccd_handle;
    cp_init.disconnect_on_fail             = false;
    cp_init.evt_handler                    = on_conn_params_evt;
    cp_init.error_handler                  = conn_params_error_handler;
    
    err_code = ble_conn_params_init(&cp_init);
    APP_ERROR_CHECK(err_code);
}*/

static void conn_params_init(void)
{
    uint32_t               err_code;
    ble_conn_params_init_t cp_init;
    
    memset(&cp_init, 0, sizeof(cp_init));

    cp_init.p_conn_params                  = NULL;
    cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
    cp_init.next_conn_params_update_delay  = NEXT_CONN_PARAMS_UPDATE_DELAY;
    cp_init.max_conn_params_update_count   = MAX_CONN_PARAMS_UPDATE_COUNT;
    cp_init.start_on_notify_cccd_handle    = BLE_GATT_HANDLE_INVALID;
    cp_init.disconnect_on_fail             = false;
    cp_init.evt_handler                    = on_conn_params_evt;
    cp_init.error_handler                  = conn_params_error_handler;
    
    err_code = ble_conn_params_init(&cp_init);
    APP_ERROR_CHECK(err_code);
}

/**@brief Initialize services that will be used by the application.
 *
 * @details Initialize the UART based data transmission by BLE.
 */
/*uint32_t ble_falco_motordata_send(ble_hrs_t * p_hrs, uint16_t heart_rate)
{
    uint32_t err_code;

    // Send value if connected and notifying
    if (p_hrs->conn_handle != BLE_CONN_HANDLE_INVALID)
    {
        uint8_t                encoded_hrm[8];//[MAX_HRM_LEN];
        uint16_t               len;
        uint16_t               hvx_len;
        ble_gatts_hvx_params_t hvx_params;

        //len     = hrm_encode(p_hrs, heart_rate, encoded_hrm);
				len=0;
        hvx_len = len;

        memset(&hvx_params, 0, sizeof(hvx_params));

        hvx_params.handle = p_hrs->hrm_handles.value_handle;
        hvx_params.type   = BLE_GATT_HVX_NOTIFICATION;
        hvx_params.offset = 0;
        hvx_params.p_len  = &hvx_len;
        hvx_params.p_data = encoded_hrm;

        err_code = sd_ble_gatts_hvx(p_hrs->conn_handle, &hvx_params);
        if ((err_code == NRF_SUCCESS) && (hvx_len != len))
        {
            err_code = NRF_ERROR_DATA_SIZE;
        }
    }
    else
    {
        err_code = NRF_ERROR_INVALID_STATE;
    }

    return err_code;
		
		
		
}*/






/**@brief Initialize the ANT HRM reception.
 */
static void ant_hrm_rx_init(void)
{
    uint32_t err_code;
    
    err_code = ant_stack_static_config();
    APP_ERROR_CHECK(err_code);
	
    err_code = sd_ant_network_address_set(ANTPLUS_NETWORK_NUMBER, m_ant_network_key);
    APP_ERROR_CHECK(err_code);
    
    err_code = sd_ant_channel_assign(ANT_HRMRX_ANT_CHANNEL,
                                     ANT_HRMRX_CHANNEL_TYPE,
                                     ANTPLUS_NETWORK_NUMBER,
                                     ANT_HRMRX_EXT_ASSIGN);
    APP_ERROR_CHECK(err_code);

    err_code = sd_ant_channel_id_set(ANT_HRMRX_ANT_CHANNEL,
                                     ANT_HRMRX_DEVICE_NUMBER,
                                     ANT_HRMRX_DEVICE_TYPE,
                                     ANT_HRMRX_TRANS_TYPE);
    APP_ERROR_CHECK(err_code);
    
    err_code = sd_ant_channel_radio_freq_set(ANT_HRMRX_ANT_CHANNEL, ANTPLUS_RF_FREQ);
    APP_ERROR_CHECK(err_code);
    
    err_code = sd_ant_channel_period_set(ANT_HRMRX_ANT_CHANNEL, ANT_HRMRX_MSG_PERIOD);
    APP_ERROR_CHECK(err_code);
}


/**@brief Handle received ANT data message.
 * 
 * @param[in]  p_evt_buffer   The buffer containing received data. 
 */
static void ant_data_messages_handle(uint8_t * p_evt_buffer)
{
    static uint32_t s_previous_beat_count = 0;    // Heart beat count from previously received page
    uint32_t        err_code;
    uint32_t        current_page;
    uint8_t         beat_count;
    uint8_t         computed_heart_rate;
    uint16_t        beat_time;

    // Decode the default page data present in all pages
    beat_time           = uint16_decode(&p_evt_buffer[ANT_BUFFER_INDEX_MESG_DATA + 4]);
    beat_count          = (uint8_t)p_evt_buffer[ANT_BUFFER_INDEX_MESG_DATA + 6];
    computed_heart_rate = (uint8_t)p_evt_buffer[ANT_BUFFER_INDEX_MESG_DATA + 7];

    // Decode page specific data
    current_page = p_evt_buffer[ANT_BUFFER_INDEX_MESG_DATA];
    switch (current_page & ~ANT_HRM_TOGGLE_MASK)
    {
        case ANT_HRM_PAGE_4:
            // Ensure that there is only one beat between time intervals.
            if ((beat_count - s_previous_beat_count) == 1)
            {
                uint16_t prev_beat = uint16_decode(&p_evt_buffer[ANT_BUFFER_INDEX_MESG_DATA + 2]);
                
                // Subtracting the event time gives the R-R interval
                ble_hrs_rr_interval_add(&m_hrs, beat_time - prev_beat);
            }

            s_previous_beat_count = beat_count;
            break;
          
        case ANT_HRM_PAGE_0:
        case ANT_HRM_PAGE_1:
        case ANT_HRM_PAGE_2:
        case ANT_HRM_PAGE_3:
        default:
            // No implementation needed.
            break;
    }
    
    // Notify the received heart rate measurement
    err_code = ble_hrs_heart_rate_measurement_send(&m_hrs, computed_heart_rate);
    if (
        (err_code != NRF_SUCCESS)
        &&
        (err_code != NRF_ERROR_INVALID_STATE)
        &&
        (err_code != BLE_ERROR_NO_TX_BUFFERS)
        &&
        (err_code != BLE_ERROR_GATTS_SYS_ATTR_MISSING)
    )
    {
        APP_ERROR_HANDLER(err_code);
    }
}

static void Falco_ant_data_messages_handle(uint8_t * p_evt_buffer)
{
	uint32_t i=0,a;
	//uint8_t AntDevID_New,TsDevID_New;
	
		while(app_uart_put(message_head[0]) != NRF_SUCCESS); // transmit data via UART 
		nrf_delay_ms(100);
	
	 for (i = 0; i < 8; i++)
			{
																					
					while(app_uart_put(p_evt_buffer[ANT_BUFFER_INDEX_MESG_DATA+i]) != NRF_SUCCESS); // transmit data via UART 
									//nrf_delay_ms(50);
				
								//	falcorx_data[i]	= (uint8_t)p_evt_buffer[ANT_BUFFER_INDEX_MESG_DATA + i];
						
								
			}
			
				/*if((p_evt_buffer[ANT_BUFFER_INDEX_MESG_DATA+0])==0x53)
				{
						//switch (p_evt_buffer[ANT_BUFFER_INDEX_MESG_DATA+1])
						{
							case 1:
											AntDevID_New =(uint32_t) p_evt_buffer[ANT_BUFFER_INDEX_MESG_DATA+2];
											break;
									
							case 2:
											TsDevID_New =(uint32_t) p_evt_buffer[ANT_BUFFER_INDEX_MESG_DATA+2];
											break;
							
							default :
								        break;
						}	//
						
					
					if((p_evt_buffer[ANT_BUFFER_INDEX_MESG_DATA+1])==1) 
						AntDevID_New = p_evt_buffer[ANT_BUFFER_INDEX_MESG_DATA+2];//store_buffer[3]// set the new device ID for the motor 24/04/15					
				else if((p_evt_buffer[ANT_BUFFER_INDEX_MESG_DATA+1])==2)
						TsDevID_New = p_evt_buffer[ANT_BUFFER_INDEX_MESG_DATA+2];//store_buffer[6]// set the new device ID for the TS-CH  01/09/15
			  }*/
			
			
}	

/**@brief ANT RX event handler.
 *
 * @param[in]   p_ant_evt   Event received from the stack.
 */
static void on_ant_evt_rx(ant_evt_t * p_ant_evt)
{
    uint32_t message_id = p_ant_evt->evt_buffer[ANT_BUFFER_INDEX_MESG_ID];
	  
    switch (message_id)
    {
        case MESG_BROADCAST_DATA_ID:
							//Falco_ant_data_messages_handle(p_ant_evt->evt_buffer);// send data to falco-motor 
							//break;																				
				
        case MESG_ACKNOWLEDGED_DATA_ID:
            //ant_data_messages_handle(p_ant_evt->evt_buffer);// send HRT-data to the BLE devices  
							Falco_ant_data_messages_handle(p_ant_evt->evt_buffer);// send data to falco-motor
            break;

        default:
            // No implementation needed.
            break;
    }
}

/**@brief ANT TX event handler.
 *
 * @param[in]   p_ant_evt   Event received from the stack.
 */
static void on_ant_evt_tx(ant_evt_t * p_ant_evt)
{
	uint32_t err_code;
	
     // Assign a new value to the broadcast data. 
            m_broadcast_data[BROADCAST_DATA_BUFFER_SIZE - 1] = m_counter;
	  
    switch (p_ant_evt->channel)
    {
        case ANT_FALCOTX_ANT_CHANNEL:			// Tx of motor data 				 
								
            
            // Broadcast the data in motor channel 
            /*err_code = sd_ant_broadcast_message_tx(ANT_FALCOTX_ANT_CHANNEL, 
                                                   BROADCAST_DATA_BUFFER_SIZE, 
                                                   m_broadcast_data);*/
				
							//if (m_rx_byter[0]!=0x84)
							err_code = sd_ant_broadcast_message_tx(ANT_FALCOTX_ANT_CHANNEL, 
                                                   BROADCAST_DATA_BUFFER_SIZE, 
                                                   m_rx_byter1);
            APP_ERROR_CHECK(err_code);
				
						
            // Increment the counter.
            //--m_counter++;
            
            err_code = bsp_indication_set(BSP_INDICATE_SENT_OK);
            APP_ERROR_CHECK(err_code);
            break;	
				
							//break;																				
				
        case ANT_FALCOTTX_ANT_CHANNEL: // Tx of the TS data 
					
						// Broadcast the data in TS channel  
            /*err_code = sd_ant_broadcast_message_tx(ANT_FALCOTTX_ANT_CHANNEL, 
                                                   BROADCAST_DATA_BUFFER_SIZE, 
                                                   m_broadcast_data);*/
				
					//if (m_rx_byter[0]==0x84)	
					 err_code = sd_ant_broadcast_message_tx(ANT_FALCOTTX_ANT_CHANNEL, 
                                                   BROADCAST_DATA_BUFFER_SIZE, 
                                                   m_rx_byter2);	
				
            APP_ERROR_CHECK(err_code);
				
						 // Increment the counter.
            //--m_counter++;
            
            err_code = bsp_indication_set(BSP_INDICATE_SENT_OK);
            APP_ERROR_CHECK(err_code);
            
            break;

        default:
            // No implementation needed.
            break;
    }
}

/**@brief Function for setting up the ANT module to be ready for MOTOR DATA TX broadcast.
 *
 * The following commands are issued in this order:
 * - assign channel 
 * - set channel ID 
 * - open channel 
 */
static void ant_channel_tx_broadcast_setup(void)
{
    uint32_t err_code,i;
	
	  err_code = ant_stack_static_config();
    APP_ERROR_CHECK(err_code);
	
    err_code = sd_ant_network_address_set(ANTPLUS_NETWORK_NUMBER, m_ant_network_key);
	  
    APP_ERROR_CHECK(err_code);
    //nrf_delay_ms(100);
	
    // Set Channel Number. 
    err_code = sd_ant_channel_assign(ANT_FALCOTX_ANT_CHANNEL, 
                                     ANT_FALCOTX_CHANNEL_TYPE, 
                                     ANT_CHANNEL_DEFAULT_NETWORK, 
                                     ANT_FALCOTX_EXT_ASSIGN);
    
    APP_ERROR_CHECK(err_code);

    // Set Channel ID. 
	
		
	
	 /*for (i = 0; i < 9; i++)
			{
					while(app_uart_put(Set_ID[i]) != NRF_SUCCESS); // transmit data via UART 
					nrf_delay_ms(50);			
								
			}
			
			while (	m_rx_byter[0]!=0x99);			
			AntDevID_New = m_rx_byter[1];*/
	
		if((AntDevID_New!=0) && (AntDevID_New!=ANT_FALCOTX_DEVICE_NUMBER))
		{
			err_code = sd_ant_channel_id_set(ANT_FALCOTX_ANT_CHANNEL, 
                                     AntDevID_New, 
                                     ANT_FALCOTX_DEVICE_TYPE, 
                                     ANT_FALCOTX_TRANS_TYPE);
		}
		else 
		{		
		
    err_code = sd_ant_channel_id_set(ANT_FALCOTX_ANT_CHANNEL, 
                                     ANT_FALCOTX_DEVICE_NUMBER, 
                                     ANT_FALCOTX_DEVICE_TYPE, 
                                     ANT_FALCOTX_TRANS_TYPE);
		}	
			
			
			
    APP_ERROR_CHECK(err_code);
		
		//Set channel frequency 
		err_code = sd_ant_channel_radio_freq_set(ANT_FALCOTX_ANT_CHANNEL, ANTPLUS_FALCOMRF_FREQ);
    APP_ERROR_CHECK(err_code);
    
		//set channel period 
    err_code = sd_ant_channel_period_set(ANT_HRMRX_ANT_CHANNEL, ANT_FALCOTX_MSG_PERIOD);
    APP_ERROR_CHECK(err_code);
		
		//set channel power
		l_power = RADIO_TXPOWER_TXPOWER_Neg12dBm ;//For External 10 dBm                           =20dbm PA+(-12dbm)=10dbm
 
		//l_power = RADIO_TXPOWER_TXPOWER_Neg8dBm //For External 14 dbm                              =20dbm PA+(-8dbm)=14dbm
			
			err_code = sd_ant_channel_radio_tx_power_set(ANT_FALCOTX_ANT_CHANNEL,RADIO_TX_POWER_LVL_CUSTOM,l_power);
			APP_ERROR_CHECK(err_code);

    // Open channel. 
    err_code = sd_ant_channel_open(ANT_FALCOTX_ANT_CHANNEL);
    APP_ERROR_CHECK(err_code);
}

/**@brief Function for setting up the ANT module to be ready for TS SENSOR TX broadcast.
 *
 * The following commands are issued in this order:
 * - assign channel for TS sensor
 * - set channel ID for TS sensor 
 * - open channel   for TS sensor 
 */
static void ant_TSchannel_tx_broadcast_setup(void)
{
    uint32_t err_code;
	
		/*err_code = ant_stack_static_config();
    APP_ERROR_CHECK(err_code);*/
	
	
	
    //err_code = sd_ant_network_address_set(ANTPLUS_NETWORK_NUMBER, m_ant_network_key);
    //APP_ERROR_CHECK(err_code);
	
    //nrf_delay_ms(100);
    // Set Channel Number. 
    err_code = sd_ant_channel_assign(ANT_FALCOTTX_ANT_CHANNEL, 
                                     ANT_FALCOTTX_CHANNEL_TYPE, 
                                     ANT_CHANNEL_DEFAULT_NETWORK, 
                                     ANT_FALCOTTX_EXT_ASSIGN);
    
    APP_ERROR_CHECK(err_code);

    // Set Channel ID. 
    err_code = sd_ant_channel_id_set(ANT_FALCOTTX_ANT_CHANNEL, 
                                     ANT_FALCOTTX_DEVICE_NUMBER, 
                                     ANT_FALCOTTX_DEVICE_TYPE, 
                                     ANT_FALCOTTX_TRANS_TYPE);
    APP_ERROR_CHECK(err_code);
		
		//Set channel frequency 
		err_code = sd_ant_channel_radio_freq_set(ANT_FALCOTTX_ANT_CHANNEL, ANTPLUS_FALCOTRF_FREQ);
    APP_ERROR_CHECK(err_code);
    
		//set channel period 
    err_code = sd_ant_channel_period_set(ANT_FALCOTTX_ANT_CHANNEL, ANT_FALCOTTX_MSG_PERIOD);
    APP_ERROR_CHECK(err_code);
		
		//set channel power
		/*l_power = RADIO_TXPOWER_TXPOWER_Neg12dBm ;//For External 10 dBm                           =20dbm PA+(-12dbm)=10dbm
 
		//l_power = RADIO_TXPOWER_TXPOWER_Neg8dBm //For External 14 dbm                              =20dbm PA+(-8dbm)=14dbm
			
			err_code = sd_ant_channel_radio_tx_power_set(ANT_FALCOTX_ANT_CHANNEL,RADIO_TX_POWER_LVL_CUSTOM,l_power);
			APP_ERROR_CHECK(err_code);*/



    // Open channel. 
    err_code = sd_ant_channel_open(ANT_FALCOTTX_ANT_CHANNEL);
    APP_ERROR_CHECK(err_code);
}

/**@brief Function for handling ANT TX channel events. 
 *
 * @param[in] event The received ANT event to handle.
 */
static void channel_event_handle(uint32_t event)
{
    uint32_t err_code;
	
	
    switch (event)
    {
        // ANT broadcast success.
        // Send a new broadcast and increment the counter.
			
        case EVENT_TX:
            
            // Assign a new value to the broadcast data. 
            m_broadcast_data[BROADCAST_DATA_BUFFER_SIZE - 1] = m_counter;
            
            // Broadcast the data in motor channel 
            err_code = sd_ant_broadcast_message_tx(ANT_FALCOTX_ANT_CHANNEL, 
                                                   BROADCAST_DATA_BUFFER_SIZE, 
                                                   m_broadcast_data);
            APP_ERROR_CHECK(err_code);
				
						// Broadcast the data in TS channel  
            err_code = sd_ant_broadcast_message_tx(ANT_FALCOTTX_ANT_CHANNEL, 
                                                   BROADCAST_DATA_BUFFER_SIZE, 
                                                   m_broadcast_data);
            APP_ERROR_CHECK(err_code);
            
            // Increment the counter.
            m_counter++;
            
            err_code = bsp_indication_set(BSP_INDICATE_SENT_OK);
            APP_ERROR_CHECK(err_code);
            break;
            
        default:
            break;
    }
}


/**@brief ANT CHANNEL_CLOSED event handler.
 */
static void on_ant_evt_channel_closed(void)
{
    ant_and_adv_start();
}


/**@brief Application's Stack ANT event handler.
 *
 * @param[in]   p_ant_evt   Event received from the stack.
 */
static void on_ant_evt(ant_evt_t * p_ant_evt)
{
	uint32_t err_code;
    /*if (p_ant_evt->channel == ANT_HRMRX_ANT_CHANNEL)// oeration of the HRT channel
    {
        switch (p_ant_evt->event)
        {
            case EVENT_RX:
                on_ant_evt_rx(p_ant_evt);
                break;

            case EVENT_CHANNEL_CLOSED:
                on_ant_evt_channel_closed();
                break;

            default:
                // No implementation needed.
                break;
        }
    }*/
		
		if (p_ant_evt->channel == ANT_FALCOTX_ANT_CHANNEL) // operation of the falco-motor channel 
		{
			 switch (p_ant_evt->event)
        {
						case EVENT_TX: // if transmisson 
                 //channel_event_handle(event); // boradcast the Hx_M motor  data  
									on_ant_evt_tx(p_ant_evt);		
                 break;	
					
            case EVENT_RX:
                on_ant_evt_rx(p_ant_evt);
                break;

            case EVENT_CHANNEL_CLOSED:
                on_ant_evt_channel_closed();
								err_code = sd_ant_channel_open(ANT_FALCOTX_ANT_CHANNEL); // open the Motor CH0 
								APP_ERROR_CHECK(err_code);
	            
              
                break;

            default:
                // No implementation needed.
                break;
        }
			
			
		}
   else if (p_ant_evt->channel == ANT_FALCOTTX_ANT_CHANNEL) // operation of the falco-motor channel 
		{
			 switch (p_ant_evt->event)
        {
						case EVENT_TX: // if transmisson 
                 //channel_event_handle(event); // boradcast the Hx_M TS sensor data
									on_ant_evt_tx(p_ant_evt);	
                 break;
						
            case EVENT_RX:
                on_ant_evt_rx(p_ant_evt);
                break;

            case EVENT_CHANNEL_CLOSED:
                on_ant_evt_channel_closed();
						
								err_code = sd_ant_channel_open(ANT_FALCOTTX_ANT_CHANNEL);// open the TS CH1 
								APP_ERROR_CHECK(err_code);				
							
                break;

            default:
                // No implementation needed.
                break;
        }
			
			
		}		
		
		
}


/**@brief Application's Stack BLE event handler.
 *
 * @param[in]   p_ble_evt   Bluetooth stack event.
 */
static void on_ble_evt(ble_evt_t * p_ble_evt)
{
    uint32_t err_code = NRF_SUCCESS;

    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED:
            err_code = bsp_indication_set(BSP_INDICATE_CONNECTED);
            APP_ERROR_CHECK(err_code);
            m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
            break;
            
        case BLE_GAP_EVT_DISCONNECTED:
            err_code = bsp_indication_set(BSP_INDICATE_IDLE);
            APP_ERROR_CHECK(err_code);

            // Need to close the ANT channel to make it safe to write bonding information to flash
				
            /*err_code = sd_ant_channel_close(ANT_HRMRX_ANT_CHANNEL);
            APP_ERROR_CHECK(err_code);*/
							
							err_code = sd_ant_channel_close(ANT_FALCOTX_ANT_CHANNEL); // close the Motor CH0 
              APP_ERROR_CHECK(err_code);
	            
              err_code = sd_ant_channel_close(ANT_FALCOTTX_ANT_CHANNEL);// close the TS CH1 
              APP_ERROR_CHECK(err_code);				
							

            // Note: Bonding information will be stored, advertising will be restarted and the
            //       ANT channel will be reopened when ANT event CHANNEL_CLOSED is received.
            break;

#ifndef BONDING_ENABLE
        case BLE_GAP_EVT_SEC_PARAMS_REQUEST:
            err_code = sd_ble_gap_sec_params_reply(m_conn_handle, 
                                                   BLE_GAP_SEC_STATUS_PAIRING_NOT_SUPP, 
                                                   NULL,
                                                   NULL);
            APP_ERROR_CHECK(err_code);
            break;
#endif // BONDING_ENABLE

        case BLE_GAP_EVT_TIMEOUT:
            if (p_ble_evt->evt.gap_evt.params.timeout.src == BLE_GAP_TIMEOUT_SRC_ADVERTISING)
            { 
                err_code = bsp_indication_set(BSP_INDICATE_IDLE);
                APP_ERROR_CHECK(err_code);
							
                // err_code = bsp_buttons_enable((1 << WAKEUP_BUTTON_ID) | (1 << BOND_DELETE_ALL_BUTTON_ID));
                // APP_ERROR_CHECK(err_code);
							
                // Go to system-off mode (this function will not return; wakeup will cause a reset)
                /*err_code = sd_power_system_off(); // remove for continous ANT+ transmission 
                APP_ERROR_CHECK(err_code);*/
            }
            break;

#ifndef	BONDING_ENABLE
            case BLE_GATTS_EVT_SYS_ATTR_MISSING:
                err_code = sd_ble_gatts_sys_attr_set(m_conn_handle,
                                                     NULL,
                                                     0,
                                                     BLE_GATTS_SYS_ATTR_FLAG_SYS_SRVCS | BLE_GATTS_SYS_ATTR_FLAG_USR_SRVCS);
                APP_ERROR_CHECK(err_code);
                break;
#endif // BONDING_ENABLE
            
        default:
            // No implementation needed.
            break;
    }
}


/**@brief Dispatches a stack event to all modules with a stack BLE event handler.
 *
 * @details This function is called from the Stack event interrupt handler after a stack BLE
 *          event has been received.
 *
 * @param[in]   p_ble_evt   Stack Bluetooth event.
 */
static void ble_evt_dispatch(ble_evt_t * p_ble_evt)
{
#ifdef BONDING_ENABLE
    dm_ble_evt_handler(p_ble_evt);
#endif // BONDING_ENABLE

    //ble_hrs_on_ble_evt(&m_hrs, p_ble_evt);// connect to HRT 
		ble_nus_on_ble_evt(&m_nus, p_ble_evt); // connect to UART 
    ble_conn_params_on_ble_evt(p_ble_evt);
    on_ble_evt(p_ble_evt);
}


#ifdef BONDING_ENABLE 
/**@brief Function for dispatching a system event to interested modules.
 *
 * @details This function is called from the System event interrupt handler after a system
 *          event has been received.
 *
 * @param[in]   sys_evt   System stack event.
 */
static void sys_evt_dispatch(uint32_t sys_evt)
{
    uint32_t err_code;
    uint32_t count;

    pstorage_sys_event_handler(sys_evt);

    // Verify if BLE bond data storage access is in progress or not.    
    if (m_app_initialized == true)
    {
        err_code = pstorage_access_status_get(&count);
        if ((err_code == NRF_SUCCESS) && (count == 0))
        {        
            ant_and_adv_start();
        }
    }
    else
    {
        m_app_initialized = true;
    }
}


/**@brief Function for handling the Device Manager events.
 *
 * @param[in]   p_evt   Data associated to the device manager event.
 */
static uint32_t device_manager_evt_handler(dm_handle_t const    * p_handle,
                                           dm_event_t const     * p_event,
                                           ret_code_t             event_result)
{
    APP_ERROR_CHECK(event_result);
    return NRF_SUCCESS;
}


/**@brief Function for the Device Manager initialization.
 */
static void device_manager_init(void)
{
    uint32_t                err_code;
    dm_init_param_t         init_data;
    dm_application_param_t  register_param;
    
    // Initialize persistent storage module.
    err_code = pstorage_init();
    APP_ERROR_CHECK(err_code);

    // Clear all bonded centrals if the Bonds Delete button is pushed.
    err_code = bsp_button_is_pressed(BOND_DELETE_ALL_BUTTON_ID,&(init_data.clear_persistent_data));
    APP_ERROR_CHECK(err_code);

    err_code = dm_init(&init_data);
    APP_ERROR_CHECK(err_code);

    memset(&register_param.sec_param, 0, sizeof(ble_gap_sec_params_t));
    
    register_param.sec_param.timeout      = SEC_PARAM_TIMEOUT;
    register_param.sec_param.bond         = SEC_PARAM_BOND;
    register_param.sec_param.mitm         = SEC_PARAM_MITM;
    register_param.sec_param.io_caps      = SEC_PARAM_IO_CAPABILITIES;
    register_param.sec_param.oob          = SEC_PARAM_OOB;
    register_param.sec_param.min_key_size = SEC_PARAM_MIN_KEY_SIZE;
    register_param.sec_param.max_key_size = SEC_PARAM_MAX_KEY_SIZE;
    register_param.evt_handler            = device_manager_evt_handler;
    register_param.service_type           = DM_PROTOCOL_CNTXT_GATT_SRVR_ID;

    err_code = dm_register(&m_app_handle, &register_param);
    APP_ERROR_CHECK(err_code);
}
#endif // BONDING_ENABLE


/**@brief BLE + ANT stack initialization.
 *
 * @details Initializes the SoftDevice and the stack event interrupt.
 */
static void ble_ant_stack_init(void)
{
    uint32_t                err_code;

    // Initialize SoftDevice
    SOFTDEVICE_HANDLER_INIT(NRF_CLOCK_LFCLKSRC_XTAL_20_PPM, NULL);
	  //SOFTDEVICE_HANDLER_INIT(NRF_CLOCK_LFCLKSRC_RC_250_PPM_250MS_CALIBRATION, NULL);
	
    
    // Initialize BLE stack
    ble_enable_params_t ble_enable_params;
    memset(&ble_enable_params, 0, sizeof(ble_enable_params));
    ble_enable_params.gatts_enable_params.service_changed = IS_SRVC_CHANGED_CHARACT_PRESENT;
    err_code = sd_ble_enable(&ble_enable_params);
    APP_ERROR_CHECK(err_code);

    // Subscribe for BLE events.
    err_code = softdevice_ble_evt_handler_set(ble_evt_dispatch);
    APP_ERROR_CHECK(err_code);
        
    // Subscribe for ANT events.
    err_code = softdevice_ant_evt_handler_set(on_ant_evt);
    APP_ERROR_CHECK(err_code);

#ifdef BONDING_ENABLE
    // Register with the SoftDevice handler module for BLE events.
    err_code = softdevice_sys_evt_handler_set(sys_evt_dispatch);
    APP_ERROR_CHECK(err_code);
#endif // BONDING_ENABLE
}


/**@brief Power manager.
 */
static void power_manage(void)
{
    uint32_t err_code;
    
    // Wait for events    
    err_code = sd_app_evt_wait();
    APP_ERROR_CHECK(err_code);
}

//--------------------------UART APPLICATION---------------------------------------------------------

void uart_error_handle(app_uart_evt_t * p_event)
{
    if (p_event->evt_type == APP_UART_COMMUNICATION_ERROR)
    {
        APP_ERROR_HANDLER(p_event->data.error_communication);
    }
    else if (p_event->evt_type == APP_UART_FIFO_ERROR)
    {
        APP_ERROR_HANDLER(p_event->data.error_code);
    }
	/*	static uint8_t data_array[15];
    static uint8_t index = 0;
    uint32_t       err_code;

    switch (p_event->evt_type)
    {
        case APP_UART_DATA_READY:
           // UNUSED_VARIABLE(app_uart_get(&data_array[index]));
            //index++;

            //if ((data_array[index - 1] == '\n') || (index >= (BLE_NUS_MAX_DATA_LEN)))
            {
                err_code = ble_nus_string_send(&m_nus, data_array, index);
                if (err_code != NRF_ERROR_INVALID_STATE)
                {
                    APP_ERROR_CHECK(err_code);
                }
                
                index = 0;
            }//
							//if(index>15)
								index = 0;//
								
            break;
				case  APP_UART_TX_EMPTY:
							
							//while(app_uart_put(falcorx_datasend[index]) != NRF_SUCCESS); // transmit data via UART
							//app_uart_put(falcorx_datasend[index]);
				      break;
							
        case APP_UART_COMMUNICATION_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_communication);
            break;

        case APP_UART_FIFO_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_code);
            break;

        default:
            break;
    }
		
		
           UNUSED_VARIABLE(app_uart_get(&data_array[index]));
						// UNUSED_VARIABLE(app_uart_get(&data_arrayrx[index]));
		
						//data_arrayrx[index]=data_array[index];
		
            index++;

           // while(app_uart_put(falcorx_datasend[index]) != NRF_SUCCESS); // transmit data via UART
							if(index>14)
							{
								 /err_code = ble_nus_string_send(&m_nus, data_array, index);
								 //err_code2 = ble_nus_string_send(&m_nus, m_broadcast_data_new, data_ln1);
                if (err_code != NRF_ERROR_INVALID_STATE)
                {
                    APP_ERROR_CHECK(err_code);
                }/
								index = 0;
							}	*/
								
            
	
}

static void uart_init()
{
	uint32_t err_code1;
	
	nrf_delay_ms(1000);
	
	const app_uart_comm_params_t comm_params =  // declar  and intialize the MCU port pin  for the UART opeartion  
      {
          RX_PIN_NUMBER, // assinged value for the Rx-Pin3 
          TX_PIN_NUMBER, // assinged Tx pin4 of the UART
          RTS_PIN_NUMBER, // assinged RTS pin for the UART 
          CTS_PIN_NUMBER, // assinged CTS pin for the UART 
					APP_UART_FLOW_CONTROL_DISABLED,	//APP_UART_FLOW_CONTROL_ENABLED, 
          false,
          UART_BAUDRATE_BAUDRATE_Baud57600//UART_BAUDRATE_BAUDRATE_Baud38400 // assinged burd rate 38400 for UART communication 
      };
	
		//LEDS_CONFIGURE(LEDS_MASK);
    //LEDS_OFF(LEDS_MASK);
    
	APP_UART_FIFO_INIT(&comm_params,
                         UART_RX_BUF_SIZE,
                         UART_TX_BUF_SIZE,
                         uart_error_handle,
                         APP_IRQ_PRIORITY_HIGH,//APP_IRQ_PRIORITY_LOW,
                         err_code1);// intialize the UART Rx and TX buffer size , error handeler and the priority of the interrupt operation  
			
	/*APP_UART_FIFO_INIT( &comm_params,
                       UART_RX_BUF_SIZE,
                       UART_TX_BUF_SIZE,
                       uart_event_handle,
                       APP_IRQ_PRIORITY_LOW,
                       err_code1);*/		

    APP_ERROR_CHECK(err_code1);// chk the error during the intializing process 
	
}	



//#ifdef ENABLE_LOOPBACK_TEST
/** @brief Function for setting the @ref ERROR_PIN high, and then enter an infinite loop.
 */
static void show_error(void)
{
    
    LEDS_ON(LEDS_MASK);
    while(true)
    {
        // Do nothing.
    }
}


/** @brief Function for testing UART loop back. 
 *  @details Transmitts one character at a time to check if the data received from the loopback is same as the transmitted data.
 *  @note  @ref TX_PIN_NUMBER must be connected to @ref RX_PIN_NUMBER)
 */
static void uart_loopback_test()
{
    uint8_t * tx_data = (uint8_t *)("\n\rLOOPBACK_TEST\n\r"); // display the message for loop back test 
    uint8_t   rx_data;

    // Start sending one byte and see if you get the same
    for (uint32_t i = 0; i < MAX_TEST_DATA_BYTES; i++)
    {
        uint32_t err_code;
        while(app_uart_put(tx_data[i]) != NRF_SUCCESS); // transmit data via UART 

        nrf_delay_ms(100);
        err_code = app_uart_get(&rx_data); //  Received data via UART 

        if ((rx_data != tx_data[i]) || (err_code != NRF_SUCCESS)) // if error is generated 
        {
            show_error(); // dispaly error data 
        }
				m_broadcast_data_new[i]=rx_data;
				
    }
		
    return;
}

static void Falco_uartoperation()
{
	  //uint8_t * tx_data = (uint8_t *)("\n\rLOOPBACK_TEST\n\r"); // display the message for loop back test 
    uint8_t    rx_data;
	
		//--- recive data from the Motor------------------ 
	
			//for (uint32_t i = 0; i < MAX_TEST_DATA_BYTES; i++)
	
		/*for (uint32_t i = 0; i < 6; i++)
    {
        uint32_t err_code;         
        err_code = app_uart_get(&rx_data); //  Received data via UART 

       if ( (err_code != NRF_SUCCESS)) // if error is generated 
        {
            show_error(); // dispaly error data 
        }
				
				m_broadcast_data_new[i]=rx_data;
				//nrf_delay_ms(10);
    }*/

		//---- Transmit data to Hx_M motor--------------------- 	
	
    for (uint32_t i = 0; i < MAX_TEST_DATA_BYTES; i++)
    {
        
        //while(app_uart_put(falcorx_data[i]) != NRF_SUCCESS); // transmit data via UART 
			while(app_uart_put(falcorx_datasend[i]) != NRF_SUCCESS); // transmit data via UART

        nrf_delay_ms(100);       
				
    }
	
	
    
		
		
    return;
	
}	

/**@brief   Function for handling app_uart events.
 *
 * @details This function will receive a single character from the app_uart module and append it to 
 *          a string. The string will be be sent over BLE when the last character received was a 
 *          'new line' i.e '\n' (hex 0x0D) or if the string has reached a length of 
 *          @ref NUS_MAX_DATA_LENGTH.
 */
/**@snippet [Handling the data received over UART] */
void uart_event_handle(app_uart_evt_t * p_event)
{
    //static uint8_t data_array[BLE_NUS_MAX_DATA_LEN];
		static uint8_t data_array[6];
    static uint8_t index = 0;
    uint32_t       err_code;

    switch (p_event->evt_type)
    {
        case APP_UART_DATA_READY:
            UNUSED_VARIABLE(app_uart_get(&data_array[index]));
            index++;

            /*if ((data_array[index - 1] == '\n') || (index >= (BLE_NUS_MAX_DATA_LEN)))
            {
                err_code = ble_nus_string_send(&m_nus, data_array, index);
                if (err_code != NRF_ERROR_INVALID_STATE)
                {
                    APP_ERROR_CHECK(err_code);
                }
                
                index = 0;
            }*/
							if(index>6)
								index = 0;
								
            break;

        case APP_UART_COMMUNICATION_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_communication);
            break;

        case APP_UART_FIFO_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_code);
            break;

        default:
            break;
    }
}
/**@snippet [Handling the data received over UART] */

void Sru532_Antennaconfig()
{
			uint8_t ant_type;
	
			ant_type = CHIP_ANTENNA;  // select the Chip Antenna for operation 
	
			nrf_gpio_cfg_output(ANT_SEL);
			nrf_gpio_cfg_output(CSD);
			nrf_gpio_cfg_output(CPS);
		 
			if(ant_type == CHIP_ANTENNA)
			{
				nrf_gpio_pin_set(ANT_SEL);         //Chip Antenna Selected
			}
			else
			{
				nrf_gpio_pin_clear(ANT_SEL);       //uFL Antenna Selected
			}
			nrf_gpio_pin_set(CSD);
			nrf_gpio_pin_set(CPS);
		 
 
			//-----POWER CONFIGURATION--------//
 
			/*sd_ant_channel_radio_tx_power_set(channel,RADIO_TX_POWER_LVL_CUSTOM,l_power);
			 
			l_power = RADIO_TXPOWER_TXPOWER_Neg12dBm //For External 10 dBm =20dbm PA+(-12dbm)=10dbm
			 
			l_power = RADIO_TXPOWER_TXPOWER_Neg8dBm //For External 14 dbm  =20dbm PA+(-8dbm)=14dbm*/

	
}	


//#endif

//---------------------------------------------------------------------------------------------------
/**@brief Application main function.
 */
int main(void)
{
    uint32_t err_code;
		uint32_t err_code1,err_code2,err_code3;
		uint8_t ant_type;
		
	// ANT event message buffer. 
    static uint8_t event_message_buffer[ANT_EVENT_MSG_BUFFER_MIN_SIZE];
	
	//------- Initialize peripherals---------------------//
	
		LEDS_CONFIGURE(LEDS_MASK); // configure the LEDs testing port 
    LEDS_OFF(LEDS_MASK);			// turn_off the LEDs for indecation of the executation 	
		//LEDS_ON(LEDS_MASK);
    //uart_init();// initialize UART
	
		//Sru532_Antennaconfig();// Intialize the chip antenna
	
	
	//--------------ANTENNA CONFIGURATION-----------------------------------//
			ant_type = CHIP_ANTENNA;  // select the Chip Antenna for operation 
			//ant_type = 0;
	
			nrf_gpio_cfg_output(ANT_SEL);
			nrf_gpio_cfg_output(CSD);
			nrf_gpio_cfg_output(CPS);
		 
			if(ant_type == CHIP_ANTENNA)
			{
				nrf_gpio_pin_set(ANT_SEL);         //Chip Antenna Selected
			}
			else
			{
				nrf_gpio_pin_clear(ANT_SEL);       //uFL Antenna Selected
			}
			nrf_gpio_pin_set(CSD);
			nrf_gpio_pin_set(CPS);
			
			
	//-------------------ANTENNA POWER CONFIGURATION----------------------------//	
			
			/*l_power = RADIO_TXPOWER_TXPOWER_Neg12dBm ;//For External 10 dBm                           =20dbm PA+(-12dbm)=10dbm
 
			//l_power = RADIO_TXPOWER_TXPOWER_Neg8dBm //For External 14 dbm                              =20dbm PA+(-8dbm)=14dbm
			

			err_code3 = sd_ant_channel_radio_tx_power_set(ANT_FALCOTX_ANT_CHANNEL,RADIO_TX_POWER_LVL_CUSTOM,l_power);
			APP_ERROR_CHECK(err_code3);*/
				
	//---------------------BURD RATE CONFIGURATION-----------------------------//		
			
			/*nrf_gpio_cfg_output(6);//BR1
			nrf_gpio_cfg_output(9);//BR3
			nrf_gpio_cfg_output(24);//BR2*/
			//nrf_gpio_cfg_output(5);//IOSELC
			
			/*nrf_gpio_pin_set(6);
			nrf_gpio_pin_clear(9);
			nrf_gpio_pin_clear(24);*/
			//nrf_gpio_pin_set(9);
			//nrf_gpio_pin_set(24);
			//-nrf_gpio_pin_set(5);
			nrf_gpio_cfg_input(30, NRF_GPIO_PIN_PULLUP);
			
			nrf_gpio_cfg_input(10, NRF_GPIO_PIN_PULLUP);
			
			
		 
	
		nrf_delay_ms(500);
	
	/*const app_uart_comm_params_t comm_params =  // declar  and intialize the MCU port pin  for the UART opeartion  
      {
          RX_PIN_NUMBER, // assinged value for the Rx-Pin3 
          TX_PIN_NUMBER, // assinged Tx pin4 of the UART
          RTS_PIN_NUMBER, // assinged RTS pin for the UART 
          CTS_PIN_NUMBER, // assinged CTS pin for the UART 
					APP_UART_FLOW_CONTROL_DISABLED,	//APP_UART_FLOW_CONTROL_ENABLED, 
          false,
          UART_BAUDRATE_BAUDRATE_Baud57600//UART_BAUDRATE_BAUDRATE_Baud38400 // assinged burd rate 38400 for UART communication 
      };
	
		//LEDS_CONFIGURE(LEDS_MASK);
    //LEDS_OFF(LEDS_MASK);
    
	APP_UART_FIFO_INIT(&comm_params,
                         UART_RX_BUF_SIZE,
                         UART_TX_BUF_SIZE,
                         uart_error_handle,
                         APP_IRQ_PRIORITY_LOW,
                         err_code1);// intialize the UART Rx and TX buffer size , error handeler and the priority of the interrupt operation */ 
			
	/*APP_UART_FIFO_INIT( &comm_params,
                       UART_RX_BUF_SIZE,
                       UART_TX_BUF_SIZE,
                       uart_event_handle,
                       APP_IRQ_PRIORITY_LOW,
                       err_code1);*/		

    //APP_ERROR_CHECK(err_code1);// chk the error during the intializing process 
			
		//lfclk_config();	
   
    timers_init();// initalize the timer 

    // Initialize S310 SoftDevice
        ble_ant_stack_init();

    err_code = bsp_init(BSP_INIT_LED | BSP_INIT_BUTTONS, APP_TIMER_TICKS(100, APP_TIMER_PRESCALER), NULL);// get error if any during intialization of the BLE 
    APP_ERROR_CHECK(err_code);// Chk the Error generated 
    // err_code = bsp_buttons_enable((1 << WAKEUP_BUTTON_ID) | (1 << BOND_DELETE_ALL_BUTTON_ID));
    // APP_ERROR_CHECK(err_code);

    // Initialize Bluetooth stack parameters.
    gap_params_init();
    advertising_init();
    services_init();
    conn_params_init();

    // Initialize ANT HRM recieve channel-0.
    //--ant_hrm_rx_init();
		
		 // Setup Channel_0 as a TX_RX Master Only. 
		  ant_channel_tx_broadcast_setup();
		
		// Setup Channel_1 as a TX Master Only.
		  //ant_TSchannel_tx_broadcast_setup();
    
    // Write counter value to last byte of the broadcast data.
    // The last byte is chosen to get the data more visible in the end of an printout
    // on the recieving end. 
    m_broadcast_data[BROADCAST_DATA_BUFFER_SIZE - 1] = m_counter;
  
    // Initiate the broadcast loop by sending a packet on air, 
    // then start waiting for an event on this broadcast message.
    err_code = sd_ant_broadcast_message_tx(ANT_FALCOTX_ANT_CHANNEL, BROADCAST_DATA_BUFFER_SIZE, m_broadcast_data);
    APP_ERROR_CHECK(err_code);
		//err_code = sd_ant_event_get(&ant_channel, &event, event_message_buffer);// get the error message during ANT operation
		
		//--err_code = sd_ant_broadcast_message_tx(ANT_FALCOTTX_ANT_CHANNEL, BROADCAST_DATA_BUFFER_SIZE, m_broadcast_data);
    //--APP_ERROR_CHECK(err_code);
		//err_code = sd_ant_event_get(&ant_channel, &event, event_message_buffer);// get the error message during ANT operation
		
		 

#ifdef BONDING_ENABLE // chk for bonding angel enable or not 

    uint32_t count;  //  intialize the counter 

    // Initialize device manager.
    device_manager_init();    

    err_code = pstorage_access_status_get(&count);// chk the error generation during the meomery handeling 
    if ((err_code == NRF_SUCCESS) && (count == 0)) // if there is no error 
			
#endif // BONDING_ENABLE
		
    {
        ant_and_adv_start();// start receving data from ANT and Transmitt data to the BLE module 
    }
		
		
/*
#ifndef ENABLE_LOOPBACK_TEST   // if loopback testing operation for UART is enabled 
		
    printf("\n\rStart: \n\r");	 // print the start operation 
		
	while (true) // contineous monitor the Rx and Tx data 
    {
        uint8_t cr;
        while(app_uart_get(&cr) != NRF_SUCCESS); // Receive the UART data from the RX line 
        while(app_uart_put(cr) != NRF_SUCCESS); // Transmit UART data 

        if (cr == 'q' || cr == 'Q') // if receive data is Q 
        {
            printf(" \n\rExit!\n\r"); // stop the communication 

            while (true)
            {
                // Do nothing.
            }
        }
    }
#else

    // This part of the example is just for testing the loopback .
    while (true)
    {
        uart_loopback_test(); // do the loopback operation 
				power_manage();       // manage the power 
    }
//#endif	
*/	
		
		uart_init();
		
		nrf_delay_ms(500);
	
		//------Configure the MOTORID set by the USER --------------------------------//
		
		//while(1)
		{	
		for (uint8_t i = 0; i < 9; i++)
			{
					while(app_uart_put(Set_ID[i]) != NRF_SUCCESS); // transmit data via UART 
					//nrf_delay_ms(50);			
					//nrf_delay_ms(500);			
			}
			nrf_delay_ms(500);
			
			while (	m_rx_byter[1]!=0x99);			
			AntDevID_New = m_rx_byter[2];
			
			if((AntDevID_New!=0) && (AntDevID_New!=ANT_FALCOTX_DEVICE_NUMBER))
		{
			err_code = sd_ant_channel_id_set(ANT_FALCOTX_ANT_CHANNEL, 
                                     AntDevID_New, 
                                     ANT_FALCOTX_DEVICE_TYPE, 
                                     ANT_FALCOTX_TRANS_TYPE);
			APP_ERROR_CHECK(err_code);	
		}
	}		
    
	//------Configure the TSID set by the USER --------------------------------//	
		nrf_delay_ms(100);
		
		/*Set_ID[2]=0x02;// select TS channenl //removal of the TS operation for FAN application  

		for (uint8_t i = 0; i < 9; i++)
			{
					while(app_uart_put(Set_ID[i]) != NRF_SUCCESS); // transmit data via UART 
					nrf_delay_ms(50);			
								
			}
			
			while (	m_rx_byter[1]!=0x99);			
			TsDevID_New = m_rx_byter[2];
			
			if((TsDevID_New!=0) && (TsDevID_New!=ANT_FALCOTTX_DEVICE_NUMBER))
		{
			err_code = sd_ant_channel_id_set(ANT_FALCOTTX_ANT_CHANNEL, 
                                     TsDevID_New, 
                                     ANT_FALCOTTX_DEVICE_TYPE, 
                                     ANT_FALCOTTX_TRANS_TYPE);
			APP_ERROR_CHECK(err_code);
   }	*/		
    
		
    // Enter main loop.
    //for (;;)
		while (true)
    {
        //--power_manage(); // keep the power consjption less 
			
				//uart_loopback_test();// Recive data from the Falco-Motor and Sensor 
			
				/*for (uint8_t index=0;index<9;index++)
				{
					while(app_uart_get(&Rx_data) != NRF_SUCCESS); // Receive the UART data from the RX line
					m_broadcast_data[BROADCAST_DATA_BUFFER_SIZE - index] = Rx_data;
					nrf_delay_ms(10);
				}*/
			
					//Falco_uartoperation();// communicate with Falco-Motor using UART 
			
				//err_code = sd_ant_event_get(&ant_channel, &event, event_message_buffer);// get the error message during ANT operation 
			
           /* if (err_code == NRF_SUCCESS) // if there is no error 
            {
							//channel_event_handle(event); // boradcast the Motor and TS sensor data
							event=0x03;
                // Handle event. 
                switch (event) // read the transmission event 
                {
                    case EVENT_TX: // if transmisson 
                        channel_event_handle(event); // boradcast the Motor and TS sensor data  
                        break;
                        
                    default:
                        break;
                }
            }*/  

								//ble_hrs_heart_rate_measurement_send(&m_hrs, m_counter);
								//ble_falco_motordata_send(&m_hrs, m_counter);
								
								//err_code2 = ble_nus_string_send(&m_nus, m_broadcast_data, data_ln);
								err_code2 = ble_nus_string_send(&m_nus, m_rx_byter, data_ln1);
								
               /* if (err_code2 != NRF_ERROR_INVALID_STATE)
                {
                    APP_ERROR_CHECK(err_code);
                }
								nrf_delay_ms(10);*/
								
								if((m_rx_byter[0]!=0x84) && (EndOfrx_falg==1))
								{
									/*cheaksum = m_rx_byter[0];
									
									for (uint8_t index3=1;index3<8;index3++)
								  cheaksum^=m_rx_byter[index3];
									 
									if( m_rx_byter[8]==cheaksum)*/
									{
										for (uint8_t index2=0;index2<8;index2++)
											m_rx_byter1[index2]=m_rx_byter[index2];
									}
									EndOfrx_falg=0;
								}	
							else if( (m_rx_byter[0]==0x84) && (EndOfrx_falg==1))
							{
								for (uint8_t index2=0;index2<8;index2++)
											m_rx_byter2[index2]=m_rx_byter[index2];
								EndOfrx_falg=0;
							}	
								
								/*for (uint8_t index2=0;index2<8;index2++)
								{
											if (m_rx_byter[0]!=0x84)													
													m_rx_byter1[index2]=m_rx_byter[index2];
											else 
													m_rx_byter2[index2]=m_rx_byter[index2];
								}*/
								
								
    }
	


    
}

/** 
 * @}
 */
